@@ -652,6 +652,8 @@ PETSC_EXTERN PetscErrorCode TSMonitorSPSwarmSolution(TS,PetscInt,PetscReal,Vec,v
 PETSC_EXTERN PetscErrorCode TSSetEventHandler(TS,PetscInt,PetscInt[],PetscBool[],PetscErrorCode (*)(TS,PetscReal,Vec,PetscScalar[],void*),PetscErrorCode (*)(TS,PetscInt,PetscInt[],PetscReal,Vec,PetscBool,void*),void*);
 PETSC_EXTERN PetscErrorCode TSSetPostEventIntervalStep(TS,PetscReal);
 PETSC_EXTERN PetscErrorCode TSSetEventTolerances(TS,PetscReal,PetscReal[]);
+PETSC_EXTERN PetscErrorCode TSGetNumEvents(TS,PetscInt*);
+
 /*J
    TSSSPType - string with the name of TSSSP scheme.
 
@@ -986,7 +988,7 @@ PETSC_EXTERN PetscErrorCode TSDiscGradSetFormulation(TS, PetscErrorCode(*)(TS, P
 /*
        PETSc interface to Sundials
 */
-#ifdef PETSC_HAVE_SUNDIALS
+#ifdef PETSC_HAVE_SUNDIALS2
 typedef enum { SUNDIALS_ADAMS=1,SUNDIALS_BDF=2} TSSundialsLmmType;
 PETSC_EXTERN const char *const TSSundialsLmmTypes[];
 typedef enum { SUNDIALS_MODIFIED_GS = 1,SUNDIALS_CLASSICAL_GS = 2 } TSSundialsGramSchmidtType;
@@ -1035,4 +1037,6 @@ PETSC_EXTERN PetscErrorCode TSGetComputeExactError(TS, PetscErrorCode (**)(TS, V
 PETSC_EXTERN PetscErrorCode TSSetComputeExactError(TS, PetscErrorCode (*)(TS, Vec, Vec));
 PETSC_EXTERN PetscErrorCode TSComputeExactError(TS, Vec, Vec);
 PETSC_EXTERN PetscErrorCode PetscConvEstUseTS(PetscConvEst, PetscBool);
+
+PETSC_EXTERN PetscErrorCode TSSetMatStructure(TS,MatStructure);
 #endif
