@@ -10,7 +10,7 @@
 . network - network to monitor
 
   Output Parameters:
-. Monitorptr - Location to put network monitor context
+. monitorptr - Location to put network monitor context
 
   Level: intermediate
 
@@ -25,7 +25,7 @@ PetscErrorCode DMNetworkMonitorCreate(DM network,DMNetworkMonitor *monitorptr)
 
   PetscFunctionBegin;
   ierr = PetscObjectGetComm((PetscObject)network,&comm);CHKERRQ(ierr);
-  ierr = MPI_Comm_size(comm, &size);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm, &size);CHKERRMPI(ierr);
   if (size > 1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Parallel DMNetworkMonitor is not supported yet");
 
   ierr = PetscMalloc1(1,&monitor);CHKERRQ(ierr);
@@ -133,8 +133,8 @@ PetscErrorCode DMNetworkMonitorAdd(DMNetworkMonitor monitor,const char *name,Pet
   PetscInt             vStart,vEnd,eStart,eEnd;
 
   PetscFunctionBegin;
-  ierr = MPI_Comm_rank(monitor->comm, &rank);CHKERRQ(ierr);
-  ierr = MPI_Comm_size(monitor->comm, &size);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(monitor->comm, &rank);CHKERRMPI(ierr);
+  ierr = MPI_Comm_size(monitor->comm, &size);CHKERRMPI(ierr);
 
   ierr = DMNetworkGetVertexRange(monitor->network, &vStart, &vEnd);CHKERRQ(ierr);
   ierr = DMNetworkGetEdgeRange(monitor->network, &eStart, &eEnd);CHKERRQ(ierr);
@@ -201,7 +201,7 @@ PetscErrorCode DMNetworkMonitorView(DMNetworkMonitor monitor,Vec x)
   PetscFunctionBegin;
   ierr = VecGetArrayRead(x, &xx);CHKERRQ(ierr);
   for (node = monitor->firstnode; node; node = node->next) {
-    ierr = DMNetworkGetVariableGlobalOffset(monitor->network, node->element, &varoffset);CHKERRQ(ierr);
+    ierr = DMNetworkGetGlobalVecOffset(monitor->network, node->element, ALL_COMPONENTS, &varoffset);CHKERRQ(ierr);
     ierr = VecGetArray(node->v, &vv);CHKERRQ(ierr);
     start = varoffset + node->start;
     for (i = 0; i < node->nodes; i++) {
