@@ -3,17 +3,18 @@
    This file contains routines for basic map object implementation.
 */
 
-#include <petscis.h> /*I "petscis.h" I*/
-#include <petscsf.h>
+#include <petsc/private/isimpl.h> /*I "petscis.h" I*/
 
 /*@
-  PetscLayoutCreate - Allocates PetscLayout space and sets the map contents to the default.
+  PetscLayoutCreate - Allocates PetscLayout space and sets the PetscLayout contents to the default.
 
-  Collective on MPI_Comm
+  Collective
 
   Input Parameters:
-+ comm - the MPI communicator
-- map - pointer to the map
+. comm - the MPI communicator
+
+  Output Parameters:
+. map - the new PetscLayout
 
   Level: advanced
 
@@ -21,10 +22,13 @@
   Typical calling sequence
 .vb
        PetscLayoutCreate(MPI_Comm,PetscLayout *);
-       PetscLayoutSetBlockSize(PetscLayout,1);
-       PetscLayoutSetSize(PetscLayout,N) // or PetscLayoutSetLocalSize(PetscLayout,n);
+       PetscLayoutSetBlockSize(PetscLayout,bs);
+       PetscLayoutSetSize(PetscLayout,N); // or PetscLayoutSetLocalSize(PetscLayout,n);
        PetscLayoutSetUp(PetscLayout);
 .ve
+  Alternatively,
+$      PetscLayoutCreateFromSizes(comm,n,N,bs,&layout);
+
   Optionally use any of the following:
 
 + PetscLayoutGetSize(PetscLayout,PetscInt *);
@@ -37,7 +41,8 @@
   user codes unless you really gain something in their use.
 
 .seealso: PetscLayoutSetLocalSize(), PetscLayoutSetSize(), PetscLayoutGetSize(), PetscLayoutGetLocalSize(), PetscLayout, PetscLayoutDestroy(),
-          PetscLayoutGetRange(), PetscLayoutGetRanges(), PetscLayoutSetBlockSize(), PetscLayoutGetBlockSize(), PetscLayoutSetUp()
+          PetscLayoutGetRange(), PetscLayoutGetRanges(), PetscLayoutSetBlockSize(), PetscLayoutGetBlockSize(), PetscLayoutSetUp(),
+          PetscLayoutCreateFromSizes()
 
 @*/
 PetscErrorCode PetscLayoutCreate(MPI_Comm comm,PetscLayout *map)
@@ -46,21 +51,70 @@ PetscErrorCode PetscLayoutCreate(MPI_Comm comm,PetscLayout *map)
 
   PetscFunctionBegin;
   ierr = PetscNew(map);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm, &(*map)->size);CHKERRMPI(ierr);
+  (*map)->comm        = comm;
+  (*map)->bs          = -1;
+  (*map)->n           = -1;
+  (*map)->N           = -1;
+  (*map)->range       = NULL;
+  (*map)->range_alloc = PETSC_TRUE;
+  (*map)->rstart      = 0;
+  (*map)->rend        = 0;
+  (*map)->setupcalled = PETSC_FALSE;
+  (*map)->oldn        = -1;
+  (*map)->oldN        = -1;
+  (*map)->oldbs       = -1;
+  PetscFunctionReturn(0);
+}
 
-  (*map)->comm   = comm;
-  (*map)->bs     = -1;
-  (*map)->n      = -1;
-  (*map)->N      = -1;
-  (*map)->range  = NULL;
-  (*map)->rstart = 0;
-  (*map)->rend   = 0;
+/*@
+  PetscLayoutCreateFromSizes - Allocates PetscLayout space, sets the layout sizes, and sets the layout up.
+
+  Collective
+
+  Input Parameters:
++ comm  - the MPI communicator
+. n     - the local size (or PETSC_DECIDE)
+. N     - the global size (or PETSC_DECIDE)
+- bs    - the block size (or PETSC_DECIDE)
+
+  Output Parameters:
+. map - the new PetscLayout
+
+  Level: advanced
+
+  Notes:
+$ PetscLayoutCreateFromSizes(comm,n,N,bs,&layout);
+  is a shorthand for
+.vb
+  PetscLayoutCreate(comm,&layout);
+  PetscLayoutSetLocalSize(layout,n);
+  PetscLayoutSetSize(layout,N);
+  PetscLayoutSetBlockSize(layout,bs);
+  PetscLayoutSetUp(layout);
+.ve
+
+.seealso: PetscLayoutCreate(), PetscLayoutSetLocalSize(), PetscLayoutSetSize(), PetscLayoutGetSize(), PetscLayoutGetLocalSize(), PetscLayout, PetscLayoutDestroy(),
+          PetscLayoutGetRange(), PetscLayoutGetRanges(), PetscLayoutSetBlockSize(), PetscLayoutGetBlockSize(), PetscLayoutSetUp(), PetscLayoutCreateFromRanges()
+
+@*/
+PetscErrorCode PetscLayoutCreateFromSizes(MPI_Comm comm,PetscInt n,PetscInt N,PetscInt bs,PetscLayout *map)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscLayoutCreate(comm, map);CHKERRQ(ierr);
+  ierr = PetscLayoutSetLocalSize(*map, n);CHKERRQ(ierr);
+  ierr = PetscLayoutSetSize(*map, N);CHKERRQ(ierr);
+  ierr = PetscLayoutSetBlockSize(*map, bs);CHKERRQ(ierr);
+  ierr = PetscLayoutSetUp(*map);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 /*@
   PetscLayoutDestroy - Frees a map object and frees its range if that exists.
 
-  Collective on MPI_Comm
+  Collective
 
   Input Parameters:
 . map - the PetscLayout
@@ -82,7 +136,7 @@ PetscErrorCode PetscLayoutDestroy(PetscLayout *map)
   PetscFunctionBegin;
   if (!*map) PetscFunctionReturn(0);
   if (!(*map)->refcnt--) {
-    ierr = PetscFree((*map)->range);CHKERRQ(ierr);
+    if ((*map)->range_alloc) {ierr = PetscFree((*map)->range);CHKERRQ(ierr);}
     ierr = ISLocalToGlobalMappingDestroy(&(*map)->mapping);CHKERRQ(ierr);
     ierr = PetscFree((*map));CHKERRQ(ierr);
   }
@@ -90,11 +144,76 @@ PetscErrorCode PetscLayoutDestroy(PetscLayout *map)
   PetscFunctionReturn(0);
 }
 
+/*@
+  PetscLayoutCreateFromRanges - Creates a new PetscLayout with the given ownership ranges and sets it up.
+
+  Collective
+
+  Input Parameters:
++ comm  - the MPI communicator
+. range - the array of ownership ranges for each rank with length commsize+1
+. mode  - the copy mode for range
+- bs    - the block size (or PETSC_DECIDE)
+
+  Output Parameters:
+. newmap - the new PetscLayout
+
+  Level: developer
+
+.seealso: PetscLayoutCreate(), PetscLayoutSetLocalSize(), PetscLayoutSetSize(), PetscLayoutGetSize(), PetscLayoutGetLocalSize(), PetscLayout, PetscLayoutDestroy(),
+          PetscLayoutGetRange(), PetscLayoutGetRanges(), PetscLayoutSetBlockSize(), PetscLayoutGetBlockSize(), PetscLayoutSetUp(), PetscLayoutCreateFromSizes()
+
+@*/
+PetscErrorCode PetscLayoutCreateFromRanges(MPI_Comm comm,const PetscInt range[],PetscCopyMode mode,PetscInt bs,PetscLayout *newmap)
+{
+  PetscLayout    map;
+  PetscMPIInt    rank;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = MPI_Comm_rank(comm, &rank);CHKERRMPI(ierr);
+  ierr = PetscLayoutCreate(comm, &map);CHKERRQ(ierr);
+  ierr = PetscLayoutSetBlockSize(map, bs);CHKERRQ(ierr);
+  switch (mode) {
+    case PETSC_COPY_VALUES:
+      ierr = PetscMalloc1(map->size+1, &map->range);CHKERRQ(ierr);
+      ierr = PetscArraycpy(map->range, range, map->size+1);CHKERRQ(ierr);
+      break;
+    case PETSC_USE_POINTER:
+      map->range_alloc = PETSC_FALSE;
+    default:
+      map->range = (PetscInt*) range;
+      break;
+  }
+  map->rstart = map->range[rank];
+  map->rend   = map->range[rank+1];
+  map->n      = map->rend - map->rstart;
+  map->N      = map->range[map->size];
+  if (PetscDefined(USE_DEBUG)) {  /* just check that n, N and bs are consistent */
+    PetscInt tmp;
+    ierr = MPIU_Allreduce(&map->n,&tmp,1,MPIU_INT,MPI_SUM,map->comm);CHKERRQ(ierr);
+    if (tmp != map->N) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Sum of local lengths %D does not equal global length %D, my local length %D.\nThe provided PetscLayout is wrong.",tmp,map->N,map->n);
+    if (map->bs > 1) {
+      if (map->n % map->bs) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Local size %D must be divisible by blocksize %D",map->n,map->bs);
+    }
+    if (map->bs > 1) {
+      if (map->N % map->bs) SETERRQ2(map->comm,PETSC_ERR_PLIB,"Global size %D must be divisible by blocksize %D",map->N,map->bs);
+    }
+  }
+  /* lock the layout */
+  map->setupcalled = PETSC_TRUE;
+  map->oldn = map->n;
+  map->oldN = map->N;
+  map->oldbs = map->bs;
+  *newmap = map;
+  PetscFunctionReturn(0);
+}
+
 /*@
   PetscLayoutSetUp - given a map where you have set either the global or local
                      size sets up the map so that it may be used.
 
-  Collective on MPI_Comm
+  Collective
 
   Input Parameters:
 . map - pointer to the map
@@ -109,6 +228,7 @@ $ PetscLayoutSetSize(PetscLayout,n) or PetscLayoutSetLocalSize(PetscLayout,N); o
 $ PetscLayoutSetUp(PetscLayout);
 $ PetscLayoutGetSize(PetscLayout,PetscInt *);
 
+  If range exists, and local size is not set, everything gets computed from the range.
 
   If the local size, global size are already set and range exists then this does nothing.
 
@@ -117,37 +237,43 @@ $ PetscLayoutGetSize(PetscLayout,PetscInt *);
 @*/
 PetscErrorCode PetscLayoutSetUp(PetscLayout map)
 {
-  PetscMPIInt    rank,size;
+  PetscMPIInt    rank;
   PetscInt       p;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if ((map->n >= 0) && (map->N >= 0) && (map->range)) PetscFunctionReturn(0);
+  if (map->setupcalled && (map->n != map->oldn || map->N != map->oldN)) SETERRQ4(map->comm,PETSC_ERR_ARG_WRONGSTATE,"Layout is already setup with (local=%D,global=%D), cannot call setup again with (local=%D,global=%D)", map->oldn, map->oldN, map->n, map->N);
+  if (map->setupcalled) PetscFunctionReturn(0);
 
   if (map->n > 0 && map->bs > 1) {
-    if (map->n % map->bs) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Local matrix size %D must be divisible by blocksize %D",map->n,map->bs);
+    if (map->n % map->bs) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Local size %D must be divisible by blocksize %D",map->n,map->bs);
   }
   if (map->N > 0 && map->bs > 1) {
-    if (map->N % map->bs) SETERRQ2(map->comm,PETSC_ERR_PLIB,"Global matrix size %D must be divisible by blocksize %D",map->N,map->bs);
+    if (map->N % map->bs) SETERRQ2(map->comm,PETSC_ERR_PLIB,"Global size %D must be divisible by blocksize %D",map->N,map->bs);
   }
 
-  ierr = MPI_Comm_size(map->comm, &size);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(map->comm, &rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(map->comm, &rank);CHKERRMPI(ierr);
   if (map->n > 0) map->n = map->n/PetscAbs(map->bs);
   if (map->N > 0) map->N = map->N/PetscAbs(map->bs);
   ierr = PetscSplitOwnership(map->comm,&map->n,&map->N);CHKERRQ(ierr);
   map->n = map->n*PetscAbs(map->bs);
   map->N = map->N*PetscAbs(map->bs);
   if (!map->range) {
-    ierr = PetscMalloc1(size+1, &map->range);CHKERRQ(ierr);
+    ierr = PetscMalloc1(map->size+1, &map->range);CHKERRQ(ierr);
   }
-  ierr = MPI_Allgather(&map->n, 1, MPIU_INT, map->range+1, 1, MPIU_INT, map->comm);CHKERRQ(ierr);
+  ierr = MPI_Allgather(&map->n, 1, MPIU_INT, map->range+1, 1, MPIU_INT, map->comm);CHKERRMPI(ierr);
 
   map->range[0] = 0;
-  for (p = 2; p <= size; p++) map->range[p] += map->range[p-1];
+  for (p = 2; p <= map->size; p++) map->range[p] += map->range[p-1];
 
   map->rstart = map->range[rank];
   map->rend   = map->range[rank+1];
+
+  /* lock the layout */
+  map->setupcalled = PETSC_TRUE;
+  map->oldn = map->n;
+  map->oldN = map->N;
+  map->oldbs = map->bs;
   PetscFunctionReturn(0);
 }
 
@@ -171,18 +297,17 @@ PetscErrorCode PetscLayoutSetUp(PetscLayout map)
 @*/
 PetscErrorCode PetscLayoutDuplicate(PetscLayout in,PetscLayout *out)
 {
-  PetscMPIInt    size;
   PetscErrorCode ierr;
   MPI_Comm       comm = in->comm;
 
   PetscFunctionBegin;
   ierr = PetscLayoutDestroy(out);CHKERRQ(ierr);
   ierr = PetscLayoutCreate(comm,out);CHKERRQ(ierr);
-  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
   ierr = PetscMemcpy(*out,in,sizeof(struct _n_PetscLayout));CHKERRQ(ierr);
-  ierr = PetscMalloc1(size+1,&(*out)->range);CHKERRQ(ierr);
-  ierr = PetscMemcpy((*out)->range,in->range,(size+1)*sizeof(PetscInt));CHKERRQ(ierr);
-
+  if (in->range) {
+    ierr = PetscMalloc1((*out)->size+1,&(*out)->range);CHKERRQ(ierr);
+    ierr = PetscArraycpy((*out)->range,in->range,(*out)->size+1);CHKERRQ(ierr);
+  }
   (*out)->refcnt = 0;
   PetscFunctionReturn(0);
 }
@@ -244,7 +369,7 @@ PetscErrorCode PetscLayoutSetISLocalToGlobalMapping(PetscLayout in,ISLocalToGlob
 
   PetscFunctionBegin;
   ierr = ISLocalToGlobalMappingGetBlockSize(ltog,&bs);CHKERRQ(ierr);
-  if (in->bs > 0 && in->bs != bs) SETERRQ2(in->comm,PETSC_ERR_PLIB,"Blocksize of layout %D must match that of mapping %D",in->bs,bs);
+  if (in->bs > 0 && (bs != 1) && in->bs != bs) SETERRQ2(in->comm,PETSC_ERR_PLIB,"Blocksize of layout %D must match that of mapping %D (or the latter must be 1)",in->bs,bs);
   ierr = PetscObjectReference((PetscObject)ltog);CHKERRQ(ierr);
   ierr = ISLocalToGlobalMappingDestroy(&in->mapping);CHKERRQ(ierr);
   in->mapping = ltog;
@@ -377,7 +502,7 @@ PetscErrorCode PetscLayoutSetBlockSize(PetscLayout map,PetscInt bs)
 {
   PetscFunctionBegin;
   if (bs < 0) PetscFunctionReturn(0);
-  if (map->n > 0 && map->n % bs) SETERRQ2(map->comm,PETSC_ERR_ARG_INCOMP,"Local size %D not compatible with block size %D",map->n,bs);
+  if (map->n > 0 && map->n % bs) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Local size %D not compatible with block size %D",map->n,bs);
   if (map->mapping) {
     PetscInt       obs;
     PetscErrorCode ierr;
@@ -476,46 +601,6 @@ PetscErrorCode  PetscLayoutGetRanges(PetscLayout map,const PetscInt *range[])
   PetscFunctionReturn(0);
 }
 
-/*@C
-   PetscSFSetGraphLayout - Set a parallel star forest via global indices and a PetscLayout
-
-   Collective
-
-   Input Arguments:
-+  sf - star forest
-.  layout - PetscLayout defining the global space
-.  nleaves - number of leaf vertices on the current process, each of these references a root on any process
-.  ilocal - locations of leaves in leafdata buffers, pass NULL for contiguous storage
-.  localmode - copy mode for ilocal
--  iremote - remote locations of root vertices for each leaf on the current process
-
-   Level: intermediate
-
-   Developers Note: Local indices which are the identity permutation in the range [0,nleaves) are discarded as they
-   encode contiguous storage. In such case, if localmode is PETSC_OWN_POINTER, the memory is deallocated as it is not
-   needed
-
-.seealso: PetscSFCreate(), PetscSFView(), PetscSFSetGraph(), PetscSFGetGraph()
-@*/
-PetscErrorCode PetscSFSetGraphLayout(PetscSF sf,PetscLayout layout,PetscInt nleaves,const PetscInt *ilocal,PetscCopyMode localmode,const PetscInt *iremote)
-{
-  PetscErrorCode ierr;
-  PetscInt       i,nroots;
-  PetscSFNode    *remote;
-
-  PetscFunctionBegin;
-  ierr = PetscLayoutGetLocalSize(layout,&nroots);CHKERRQ(ierr);
-  ierr = PetscMalloc1(nleaves,&remote);CHKERRQ(ierr);
-  for (i=0; i<nleaves; i++) {
-    PetscInt owner = -1;
-    ierr = PetscLayoutFindOwner(layout,iremote[i],&owner);CHKERRQ(ierr);
-    remote[i].rank  = owner;
-    remote[i].index = iremote[i] - layout->range[owner];
-  }
-  ierr = PetscSFSetGraph(sf,nroots,nleaves,ilocal,localmode,remote,PETSC_OWN_POINTER);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
 /*@
   PetscLayoutCompare - Compares two layouts
 
@@ -538,14 +623,12 @@ PetscErrorCode PetscSFSetGraphLayout(PetscSF sf,PetscLayout layout,PetscInt nlea
 PetscErrorCode PetscLayoutCompare(PetscLayout mapa,PetscLayout mapb,PetscBool *congruent)
 {
   PetscErrorCode ierr;
-  PetscMPIInt    sizea,sizeb;
 
   PetscFunctionBegin;
   *congruent = PETSC_FALSE;
-  ierr = MPI_Comm_size(mapa->comm,&sizea);CHKERRQ(ierr);
-  ierr = MPI_Comm_size(mapb->comm,&sizeb);CHKERRQ(ierr);
-  if (mapa->N == mapb->N && mapa->range && mapb->range && sizea == sizeb) {
-    ierr = PetscMemcmp(mapa->range,mapb->range,(sizea+1)*sizeof(PetscInt),congruent);CHKERRQ(ierr);
+  if (mapa->N == mapb->N && mapa->range && mapb->range && mapa->size == mapb->size) {
+    ierr = PetscArraycmp(mapa->range,mapb->range,mapa->size+1,congruent);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
+
