@@ -2,7 +2,7 @@
 /*
    This file contains routines for Parallel vector operations.
  */
-#include <petscoptions.h>
+#include <petscsys.h>
 #include <../src/vec/vec/impls/mpi/pvecimpl.h>   /*I  "petscvec.h"   I*/
 
 PetscErrorCode VecDot_MPI(Vec xin,Vec yin,PetscScalar *z)
@@ -46,7 +46,7 @@ static PetscErrorCode VecPlaceArray_MPI(Vec vin,const PetscScalar *a)
   PetscFunctionReturn(0);
 }
 
-static PetscErrorCode VecDuplicate_MPI(Vec win,Vec *v)
+PetscErrorCode VecDuplicate_MPI(Vec win,Vec *v)
 {
   PetscErrorCode ierr;
   Vec_MPI        *vw,*w = (Vec_MPI*)win->data;
@@ -56,7 +56,7 @@ static PetscErrorCode VecDuplicate_MPI(Vec win,Vec *v)
   ierr = VecCreate(PetscObjectComm((PetscObject)win),v);CHKERRQ(ierr);
   ierr = PetscLayoutReference(win->map,&(*v)->map);CHKERRQ(ierr);
 
-  ierr = VecCreate_MPI_Private(*v,PETSC_TRUE,w->nghost,0);CHKERRQ(ierr);
+  ierr = VecCreate_MPI_Private(*v,PETSC_TRUE,w->nghost,NULL);CHKERRQ(ierr);
   vw   = (Vec_MPI*)(*v)->data;
   ierr = PetscMemcpy((*v)->ops,win->ops,sizeof(struct _VecOps));CHKERRQ(ierr);
 
@@ -90,15 +90,23 @@ static PetscErrorCode VecDuplicate_MPI(Vec win,Vec *v)
 static PetscErrorCode VecSetOption_MPI(Vec V,VecOption op,PetscBool flag)
 {
   Vec_MPI        *v = (Vec_MPI*)V->data;
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
   switch (op) {
   case VEC_IGNORE_OFF_PROC_ENTRIES: V->stash.donotstash = flag;
     break;
   case VEC_IGNORE_NEGATIVE_INDICES: V->stash.ignorenegidx = flag;
     break;
-  case VEC_SUBSET_OFF_PROC_ENTRIES: v->assembly_subset = flag;
+  case VEC_SUBSET_OFF_PROC_ENTRIES:
+    v->assembly_subset = flag; /* See the same logic in MatAssembly wrt MAT_SUBSET_OFF_PROC_ENTRIES */
+    if (!v->assembly_subset) { /* User indicates "do not reuse the communication pattern" */
+      ierr = VecAssemblyReset_MPI(V);CHKERRQ(ierr); /* Reset existing pattern to free memory */
+      v->first_assembly_done = PETSC_FALSE; /* Mark the first assembly is not done */
+    }
     break;
   }
+
   PetscFunctionReturn(0);
 }
 
@@ -110,7 +118,7 @@ static PetscErrorCode VecResetArray_MPI(Vec vin)
 
   PetscFunctionBegin;
   v->array         = v->unplacedarray;
-  v->unplacedarray = 0;
+  v->unplacedarray = NULL;
   if (v->localrep) {
     ierr = VecResetArray(v->localrep);CHKERRQ(ierr);
   }
@@ -126,15 +134,17 @@ static PetscErrorCode VecAssemblySend_MPI_Private(MPI_Comm comm,const PetscMPIIn
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  /* x->recvhdr only exists when we are reusing a communication network.  In that case, some messages can be empty, but
-   * we have to send them this time if we sent them before because the receiver is expecting them. */
-  if (hdr->count || (x->recvhdr && x->sendptrs[rankid].ints)) {
-    ierr = MPI_Isend(x->sendptrs[rankid].ints,hdr->count,MPIU_INT,rank,tag[0],comm,&req[0]);CHKERRQ(ierr);
-    ierr = MPI_Isend(x->sendptrs[rankid].scalars,hdr->count,MPIU_SCALAR,rank,tag[1],comm,&req[1]);CHKERRQ(ierr);
+  /* x->first_assembly_done indicates we are reusing a communication network. In that case, some
+     messages can be empty, but we have to send them this time if we sent them before because the
+     receiver is expecting them.
+   */
+  if (hdr->count || (x->first_assembly_done && x->sendptrs[rankid].ints)) {
+    ierr = MPI_Isend(x->sendptrs[rankid].ints,hdr->count,MPIU_INT,rank,tag[0],comm,&req[0]);CHKERRMPI(ierr);
+    ierr = MPI_Isend(x->sendptrs[rankid].scalars,hdr->count,MPIU_SCALAR,rank,tag[1],comm,&req[1]);CHKERRMPI(ierr);
   }
-  if (hdr->bcount || (x->recvhdr && x->sendptrs[rankid].intb)) {
-    ierr = MPI_Isend(x->sendptrs[rankid].intb,hdr->bcount,MPIU_INT,rank,tag[2],comm,&req[2]);CHKERRQ(ierr);
-    ierr = MPI_Isend(x->sendptrs[rankid].scalarb,hdr->bcount*bs,MPIU_SCALAR,rank,tag[3],comm,&req[3]);CHKERRQ(ierr);
+  if (hdr->bcount || (x->first_assembly_done && x->sendptrs[rankid].intb)) {
+    ierr = MPI_Isend(x->sendptrs[rankid].intb,hdr->bcount,MPIU_INT,rank,tag[2],comm,&req[2]);CHKERRMPI(ierr);
+    ierr = MPI_Isend(x->sendptrs[rankid].scalarb,hdr->bcount*bs,MPIU_SCALAR,rank,tag[3],comm,&req[3]);CHKERRMPI(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -153,9 +163,9 @@ static PetscErrorCode VecAssemblyRecv_MPI_Private(MPI_Comm comm,const PetscMPIIn
 
   if (hdr->count) {
     ierr = PetscSegBufferGet(x->segrecvint,hdr->count,&frame->ints);CHKERRQ(ierr);
-    ierr = MPI_Irecv(frame->ints,hdr->count,MPIU_INT,rank,tag[0],comm,&req[0]);CHKERRQ(ierr);
+    ierr = MPI_Irecv(frame->ints,hdr->count,MPIU_INT,rank,tag[0],comm,&req[0]);CHKERRMPI(ierr);
     ierr = PetscSegBufferGet(x->segrecvscalar,hdr->count,&frame->scalars);CHKERRQ(ierr);
-    ierr = MPI_Irecv(frame->scalars,hdr->count,MPIU_SCALAR,rank,tag[1],comm,&req[1]);CHKERRQ(ierr);
+    ierr = MPI_Irecv(frame->scalars,hdr->count,MPIU_SCALAR,rank,tag[1],comm,&req[1]);CHKERRMPI(ierr);
     frame->pendings = 2;
   } else {
     frame->ints = NULL;
@@ -165,9 +175,9 @@ static PetscErrorCode VecAssemblyRecv_MPI_Private(MPI_Comm comm,const PetscMPIIn
 
   if (hdr->bcount) {
     ierr = PetscSegBufferGet(x->segrecvint,hdr->bcount,&frame->intb);CHKERRQ(ierr);
-    ierr = MPI_Irecv(frame->intb,hdr->bcount,MPIU_INT,rank,tag[2],comm,&req[2]);CHKERRQ(ierr);
+    ierr = MPI_Irecv(frame->intb,hdr->bcount,MPIU_INT,rank,tag[2],comm,&req[2]);CHKERRMPI(ierr);
     ierr = PetscSegBufferGet(x->segrecvscalar,hdr->bcount*bs,&frame->scalarb);CHKERRQ(ierr);
-    ierr = MPI_Irecv(frame->scalarb,hdr->bcount*bs,MPIU_SCALAR,rank,tag[3],comm,&req[3]);CHKERRQ(ierr);
+    ierr = MPI_Irecv(frame->scalarb,hdr->bcount*bs,MPIU_SCALAR,rank,tag[3],comm,&req[3]);CHKERRMPI(ierr);
     frame->pendingb = 2;
   } else {
     frame->intb = NULL;
@@ -189,13 +199,11 @@ static PetscErrorCode VecAssemblyBegin_MPI_BTS(Vec X)
 
   ierr = PetscObjectGetComm((PetscObject)X,&comm);CHKERRQ(ierr);
   ierr = VecGetBlockSize(X,&bs);CHKERRQ(ierr);
-#if defined(PETSC_USE_DEBUG)
-  {
+  if (PetscDefined(USE_DEBUG)) {
     InsertMode addv;
     ierr = MPIU_Allreduce((PetscEnum*)&X->stash.insertmode,(PetscEnum*)&addv,1,MPIU_ENUM,MPI_BOR,comm);CHKERRQ(ierr);
     if (addv == (ADD_VALUES|INSERT_VALUES)) SETERRQ(comm,PETSC_ERR_ARG_NOTSAMETYPE,"Some processors inserted values while others added");
   }
-#endif
   X->bstash.insertmode = X->stash.insertmode; /* Block stash implicitly tracks InsertMode of scalar stash */
 
   ierr = VecStashSortCompress_Private(&X->stash);CHKERRQ(ierr);
@@ -223,22 +231,21 @@ static PetscErrorCode VecAssemblyBegin_MPI_BTS(Vec X)
     if (X->stash.n) {
       x->sendptrs[i].ints    = &X->stash.idx[j];
       x->sendptrs[i].scalars = &X->stash.array[j];
-      for ( ; j<X->stash.n && X->stash.idx[j] < X->map->range[rank+1]; j++) x->sendhdr[i].count++;
+      for (; j<X->stash.n && X->stash.idx[j] < X->map->range[rank+1]; j++) x->sendhdr[i].count++;
     }
     x->sendhdr[i].bcount = 0;
     if (X->bstash.n) {
       x->sendptrs[i].intb    = &X->bstash.idx[jb];
       x->sendptrs[i].scalarb = &X->bstash.array[jb*bs];
-      for ( ; jb<X->bstash.n && X->bstash.idx[jb]*bs < X->map->range[rank+1]; jb++) x->sendhdr[i].bcount++;
+      for (; jb<X->bstash.n && X->bstash.idx[jb]*bs < X->map->range[rank+1]; jb++) x->sendhdr[i].bcount++;
     }
   }
 
   if (!x->segrecvint) {ierr = PetscSegBufferCreate(sizeof(PetscInt),1000,&x->segrecvint);CHKERRQ(ierr);}
   if (!x->segrecvscalar) {ierr = PetscSegBufferCreate(sizeof(PetscScalar),1000,&x->segrecvscalar);CHKERRQ(ierr);}
   if (!x->segrecvframe) {ierr = PetscSegBufferCreate(sizeof(VecAssemblyFrame),50,&x->segrecvframe);CHKERRQ(ierr);}
-  if (x->recvhdr) {             /* VEC_SUBSET_OFF_PROC_ENTRIES and this is not the first assembly */
+  if (x->first_assembly_done) { /* this is not the first assembly */
     PetscMPIInt tag[4];
-    if (!x->assembly_subset) SETERRQ(comm,PETSC_ERR_PLIB,"Attempt to reuse rendezvous when not VEC_SUBSET_OFF_PROC_ENTRIES");
     for (i=0; i<4; i++) {ierr = PetscCommGetNewTag(comm,&tag[i]);CHKERRQ(ierr);}
     for (i=0; i<x->nsendranks; i++) {
       ierr = VecAssemblySend_MPI_Private(comm,tag,i,x->sendranks[i],x->sendhdr+i,x->sendreqs+4*i,X);CHKERRQ(ierr);
@@ -247,11 +254,16 @@ static PetscErrorCode VecAssemblyBegin_MPI_BTS(Vec X)
       ierr = VecAssemblyRecv_MPI_Private(comm,tag,x->recvranks[i],x->recvhdr+i,x->recvreqs+4*i,X);CHKERRQ(ierr);
     }
     x->use_status = PETSC_TRUE;
-  } else {                      /* First time */
+  } else { /* First time assembly */
     ierr = PetscCommBuildTwoSidedFReq(comm,3,MPIU_INT,x->nsendranks,x->sendranks,(PetscInt*)x->sendhdr,&x->nrecvranks,&x->recvranks,&x->recvhdr,4,&x->sendreqs,&x->recvreqs,VecAssemblySend_MPI_Private,VecAssemblyRecv_MPI_Private,X);CHKERRQ(ierr);
     x->use_status = PETSC_FALSE;
   }
 
+  /* The first_assembly_done flag is only meaningful when x->assembly_subset is set.
+     This line says when assembly_subset is set, then we mark that the first assembly is done.
+   */
+  x->first_assembly_done = x->assembly_subset;
+
   {
     PetscInt nstash,reallocs;
     ierr = VecStashGetInfo_Private(&X->stash,&nstash,&reallocs);CHKERRQ(ierr);
@@ -279,6 +291,7 @@ static PetscErrorCode VecAssemblyEnd_MPI_BTS(Vec X)
     PetscFunctionReturn(0);
   }
 
+  if (!x->segrecvframe) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Missing segrecvframe! Probably you forgot to call VecAssemblyBegin first");
   ierr = VecGetArray(X,&xarray);CHKERRQ(ierr);
   ierr = PetscSegBufferExtractInPlace(x->segrecvframe,&frame);CHKERRQ(ierr);
   ierr = PetscMalloc2(4*x->nrecvranks,&some_indices,x->use_status?4*x->nrecvranks:0,&some_statuses);CHKERRQ(ierr);
@@ -289,7 +302,7 @@ static PetscErrorCode VecAssemblyEnd_MPI_BTS(Vec X)
      * when VEC_SUBSET_OFF_PROC_ENTRIES has not been set, because we could exchange exact sizes in the initial
      * rendezvous.  When the rendezvous is elided, however, we use MPI_Status to get actual message lengths, so that
      * subsequent assembly can set a proper subset of the values. */
-    ierr = MPI_Waitsome(4*x->nrecvranks,x->recvreqs,&ndone,some_indices,x->use_status?some_statuses:MPI_STATUSES_IGNORE);CHKERRQ(ierr);
+    ierr = MPI_Waitsome(4*x->nrecvranks,x->recvreqs,&ndone,some_indices,x->use_status?some_statuses:MPI_STATUSES_IGNORE);CHKERRMPI(ierr);
     for (ii=0; ii<ndone; ii++) {
       PetscInt i = some_indices[ii]/4,j,k;
       InsertMode imode = (InsertMode)x->recvhdr[i].insertmode;
@@ -302,7 +315,7 @@ static PetscErrorCode VecAssemblyEnd_MPI_BTS(Vec X)
         PetscMPIInt count;
         if (--frame[i].pendings > 0) continue;
         if (x->use_status) {
-          ierr = MPI_Get_count(&some_statuses[ii],intmsg ? MPIU_INT : MPIU_SCALAR,&count);CHKERRQ(ierr);
+          ierr = MPI_Get_count(&some_statuses[ii],intmsg ? MPIU_INT : MPIU_SCALAR,&count);CHKERRMPI(ierr);
         } else count = x->recvhdr[i].count;
         for (j=0,recvint=frame[i].ints,recvscalar=frame[i].scalars; j<count; j++,recvint++) {
           PetscInt loc = *recvint - X->map->rstart;
@@ -321,7 +334,7 @@ static PetscErrorCode VecAssemblyEnd_MPI_BTS(Vec X)
         PetscMPIInt count;
         if (--frame[i].pendingb > 0) continue;
         if (x->use_status) {
-          ierr = MPI_Get_count(&some_statuses[ii],intmsg ? MPIU_INT : MPIU_SCALAR,&count);CHKERRQ(ierr);
+          ierr = MPI_Get_count(&some_statuses[ii],intmsg ? MPIU_INT : MPIU_SCALAR,&count);CHKERRMPI(ierr);
           if (!intmsg) count /= bs; /* Convert from number of scalars to number of blocks */
         } else count = x->recvhdr[i].bcount;
         for (j=0,recvint=frame[i].intb,recvscalar=frame[i].scalarb; j<count; j++,recvint++) {
@@ -340,7 +353,7 @@ static PetscErrorCode VecAssemblyEnd_MPI_BTS(Vec X)
     }
   }
   ierr = VecRestoreArray(X,&xarray);CHKERRQ(ierr);
-  ierr = MPI_Waitall(4*x->nsendranks,x->sendreqs,MPI_STATUSES_IGNORE);CHKERRQ(ierr);
+  ierr = MPI_Waitall(4*x->nsendranks,x->sendreqs,MPI_STATUSES_IGNORE);CHKERRMPI(ierr);
   ierr = PetscFree2(some_indices,some_statuses);CHKERRQ(ierr);
   if (x->assembly_subset) {
     void *dummy;                /* reset segbuffers */
@@ -422,10 +435,10 @@ static struct _VecOps DvOps = { VecDuplicate_MPI, /* 1 */
                                 VecSetValues_MPI, /* 20 */
                                 VecAssemblyBegin_MPI_BTS,
                                 VecAssemblyEnd_MPI_BTS,
-                                0,
+                                NULL,
                                 VecGetSize_MPI,
                                 VecGetSize_Seq,
-                                0,
+                                NULL,
                                 VecMax_MPI,
                                 VecMin_MPI,
                                 VecSetRandom_Seq,
@@ -443,8 +456,8 @@ static struct _VecOps DvOps = { VecDuplicate_MPI, /* 1 */
                                 VecLoad_Default,
                                 VecReciprocal_Default,
                                 VecConjugate_Seq,
-                                0,
-                                0,
+                                NULL,
+                                NULL,
                                 VecResetArray_MPI,
                                 VecSetFromOptions_MPI,/*set from options */
                                 VecMaxPointwiseDivide_Seq,
@@ -452,23 +465,24 @@ static struct _VecOps DvOps = { VecDuplicate_MPI, /* 1 */
                                 VecPointwiseMaxAbs_Seq,
                                 VecPointwiseMin_Seq,
                                 VecGetValues_MPI,
-                                0,
-                                0,
-                                0,
-                                0,
-                                0,
-                                0,
+                                NULL,
+                                NULL,
+                                NULL,
+                                NULL,
+                                NULL,
+                                NULL,
                                 VecStrideGather_Default,
                                 VecStrideScatter_Default,
-                                0,
-                                0,
-                                0,
-                                0,
-                                0,
+                                NULL,
+                                NULL,
+                                NULL,
+                                NULL,
+                                NULL,
                                 VecStrideSubSetGather_Default,
                                 VecStrideSubSetScatter_Default,
-                                0,
-                                0
+                                NULL,
+                                NULL,
+                                NULL
 };
 
 /*
@@ -490,22 +504,22 @@ PetscErrorCode VecCreate_MPI_Private(Vec v,PetscBool alloc,PetscInt nghost,const
   ierr           = PetscMemcpy(v->ops,&DvOps,sizeof(DvOps));CHKERRQ(ierr);
   s->nghost      = nghost;
   v->petscnative = PETSC_TRUE;
+  if (array) v->offloadmask = PETSC_OFFLOAD_CPU;
 
   ierr = PetscLayoutSetUp(v->map);CHKERRQ(ierr);
 
   s->array           = (PetscScalar*)array;
-  s->array_allocated = 0;
+  s->array_allocated = NULL;
   if (alloc && !array) {
     PetscInt n = v->map->n+nghost;
-    ierr               = PetscMalloc1(n,&s->array);CHKERRQ(ierr);
+    ierr               = PetscCalloc1(n,&s->array);CHKERRQ(ierr);
     ierr               = PetscLogObjectMemory((PetscObject)v,n*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr               = PetscMemzero(s->array,n*sizeof(PetscScalar));CHKERRQ(ierr);
     s->array_allocated = s->array;
   }
 
   /* By default parallel vectors do not have local representation */
-  s->localrep    = 0;
-  s->localupdate = 0;
+  s->localrep    = NULL;
+  s->localupdate = NULL;
 
   v->stash.insertmode = NOT_SET_VALUES;
   v->bstash.insertmode = NOT_SET_VALUES;
@@ -531,15 +545,15 @@ PetscErrorCode VecCreate_MPI_Private(Vec v,PetscBool alloc,PetscInt nghost,const
 
   Level: beginner
 
-.seealso: VecCreate(), VecSetType(), VecSetFromOptions(), VecCreateMpiWithArray(), VECMPI, VecType, VecCreateMPI(), VecCreateMpi()
+.seealso: VecCreate(), VecSetType(), VecSetFromOptions(), VecCreateMPIWithArray(), VECMPI, VecType, VecCreateMPI(), VecCreateMPI()
 M*/
 
-PETSC_EXTERN PetscErrorCode VecCreate_MPI(Vec vv)
+PetscErrorCode VecCreate_MPI(Vec vv)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = VecCreate_MPI_Private(vv,PETSC_TRUE,0,0);CHKERRQ(ierr);
+  ierr = VecCreate_MPI_Private(vv,PETSC_TRUE,0,NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -560,7 +574,7 @@ PETSC_EXTERN PetscErrorCode VecCreate_Standard(Vec v)
   PetscMPIInt    size;
 
   PetscFunctionBegin;
-  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)v),&size);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)v),&size);CHKERRMPI(ierr);
   if (size == 1) {
     ierr = VecSetType(v,VECSEQ);CHKERRQ(ierr);
   } else {
@@ -573,7 +587,7 @@ PETSC_EXTERN PetscErrorCode VecCreate_Standard(Vec v)
    VecCreateMPIWithArray - Creates a parallel, array-style vector,
    where the user provides the array space to store the vector values.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameters:
 +  comm  - the MPI communicator to use
@@ -597,8 +611,6 @@ PETSC_EXTERN PetscErrorCode VecCreate_Standard(Vec v)
 
    Level: intermediate
 
-   Concepts: vectors^creating with array
-
 .seealso: VecCreateSeqWithArray(), VecCreate(), VecDuplicate(), VecDuplicateVecs(), VecCreateGhost(),
           VecCreateMPI(), VecCreateGhostWithArray(), VecPlaceArray()
 
@@ -621,7 +633,7 @@ PetscErrorCode  VecCreateMPIWithArray(MPI_Comm comm,PetscInt bs,PetscInt n,Petsc
    VecCreateGhostWithArray - Creates a parallel vector with ghost padding on each processor;
    the caller allocates the array space.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameters:
 +  comm - the MPI communicator to use
@@ -642,8 +654,6 @@ PetscErrorCode  VecCreateMPIWithArray(MPI_Comm comm,PetscInt bs,PetscInt n,Petsc
 
    Level: advanced
 
-   Concepts: vectors^creating with array
-   Concepts: vectors^ghosted
 
 .seealso: VecCreate(), VecGhostGetLocalForm(), VecGhostRestoreLocalForm(),
           VecCreateGhost(), VecCreateSeqWithArray(), VecCreateMPIWithArray(),
@@ -660,7 +670,7 @@ PetscErrorCode  VecCreateGhostWithArray(MPI_Comm comm,PetscInt n,PetscInt N,Pets
   PetscInt               rstart,i,*indices;
 
   PetscFunctionBegin;
-  *vv = 0;
+  *vv = NULL;
 
   if (n == PETSC_DECIDE)      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Must set local size");
   if (nghost == PETSC_DECIDE) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Must set local ghost size");
@@ -705,7 +715,7 @@ PetscErrorCode  VecCreateGhostWithArray(MPI_Comm comm,PetscInt n,PetscInt N,Pets
 /*@
    VecCreateGhost - Creates a parallel vector with ghost padding on each processor.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameters:
 +  comm - the MPI communicator to use
@@ -725,8 +735,6 @@ PetscErrorCode  VecCreateGhostWithArray(MPI_Comm comm,PetscInt n,PetscInt N,Pets
 
    Level: advanced
 
-   Concepts: vectors^ghosted
-
 .seealso: VecCreateSeq(), VecCreate(), VecDuplicate(), VecDuplicateVecs(), VecCreateMPI(),
           VecGhostGetLocalForm(), VecGhostRestoreLocalForm(), VecGhostUpdateBegin(),
           VecCreateGhostWithArray(), VecCreateMPIWithArray(), VecGhostUpdateEnd(),
@@ -738,7 +746,7 @@ PetscErrorCode  VecCreateGhost(MPI_Comm comm,PetscInt n,PetscInt N,PetscInt ngho
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = VecCreateGhostWithArray(comm,n,N,nghost,ghosts,0,vv);CHKERRQ(ierr);
+  ierr = VecCreateGhostWithArray(comm,n,N,nghost,ghosts,NULL,vv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -763,8 +771,6 @@ PetscErrorCode  VecCreateGhost(MPI_Comm comm,PetscInt n,PetscInt N,PetscInt ngho
 
    Level: advanced
 
-   Concepts: vectors^ghosted
-
 .seealso: VecCreateSeq(), VecCreate(), VecDuplicate(), VecDuplicateVecs(), VecCreateMPI(),
           VecGhostGetLocalForm(), VecGhostRestoreLocalForm(), VecGhostUpdateBegin(),
           VecCreateGhostWithArray(), VecCreateMPIWithArray(), VecGhostUpdateEnd(),
@@ -832,7 +838,7 @@ PetscErrorCode  VecMPISetGhost(Vec vv,PetscInt nghost,const PetscInt ghosts[])
    VecCreateGhostBlockWithArray - Creates a parallel vector with ghost padding on each processor;
    the caller allocates the array space. Indices in the ghost region are based on blocks.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameters:
 +  comm - the MPI communicator to use
@@ -856,8 +862,6 @@ PetscErrorCode  VecMPISetGhost(Vec vv,PetscInt nghost,const PetscInt ghosts[])
 
    Level: advanced
 
-   Concepts: vectors^creating ghosted
-   Concepts: vectors^creating with array
 
 .seealso: VecCreate(), VecGhostGetLocalForm(), VecGhostRestoreLocalForm(),
           VecCreateGhost(), VecCreateSeqWithArray(), VecCreateMPIWithArray(),
@@ -874,7 +878,7 @@ PetscErrorCode  VecCreateGhostBlockWithArray(MPI_Comm comm,PetscInt bs,PetscInt
   PetscInt               rstart,i,nb,*indices;
 
   PetscFunctionBegin;
-  *vv = 0;
+  *vv = NULL;
 
   if (n == PETSC_DECIDE)      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Must set local size");
   if (nghost == PETSC_DECIDE) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Must set local ghost size");
@@ -922,7 +926,7 @@ PetscErrorCode  VecCreateGhostBlockWithArray(MPI_Comm comm,PetscInt bs,PetscInt
    VecCreateGhostBlock - Creates a parallel vector with ghost padding on each processor.
         The indicing of the ghost points is done with blocks.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameters:
 +  comm - the MPI communicator to use
@@ -945,8 +949,6 @@ PetscErrorCode  VecCreateGhostBlockWithArray(MPI_Comm comm,PetscInt bs,PetscInt
 
    Level: advanced
 
-   Concepts: vectors^ghosted
-
 .seealso: VecCreateSeq(), VecCreate(), VecDuplicate(), VecDuplicateVecs(), VecCreateMPI(),
           VecGhostGetLocalForm(), VecGhostRestoreLocalForm(),
           VecCreateGhostWithArray(), VecCreateMPIWithArray(), VecCreateGhostBlockWithArray()
@@ -957,6 +959,6 @@ PetscErrorCode  VecCreateGhostBlock(MPI_Comm comm,PetscInt bs,PetscInt n,PetscIn
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = VecCreateGhostBlockWithArray(comm,bs,n,N,nghost,ghosts,0,vv);CHKERRQ(ierr);
+  ierr = VecCreateGhostBlockWithArray(comm,bs,n,N,nghost,ghosts,NULL,vv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
